;  Tester (Final) 
;  By Ken Lill
;  June 21, 2011
;  Version 1.0
;
;  Assembled by hand by Ken Lill.
;
;  Compares cart using Bally/Astrocade and RAM expansion unit.
;
;  Version 1.0 - June 21, 2011 - Ken Lill
;     - Programmed by Ken Lill, and disassembled from original
;       ROM file using DZ80 2.0 and Richard Degler's BallyHVG.LUA
;       script by Adam Trionfo.
;     - Put into ZMac assembler format from disassembled file with
;       lots of hand disassembly.
;     - Comments added by Adam Trionfo based on information relayed
;       via telephone to Adam.
;     - This program assembles into the EXACT bytes that were in the
;       original file.  However, the binary file must be edited by
;       hand to get all of the bytes in the correct order and to remove
;       the extra zeros that are created from $2000-$6000.  
;    
;  This code can be assembled into a usable binary using the ZMac
;  assembler.  Here is a sample of the command line required:
;
;      zmac -i -m -o tester2.bin -x tester2.lst tester2.asm         

INCLUDE "HVGLIB.H"

        ORG     $2000          ; First byte of Cartridge

        DB      $55            ; Bally system Sentinel

        DW      MENUST         ; Next menu link
        DW      PRGNAM         ; Address of title for program
        DW      PRGST          ; Jump here if prog is selected
        

PRGST:  LD      HL,LSTBYT+1    ; Set-up for LDIR - Source
        LD      DE,$6000       ; Destination (Expansion RAM)
        LD      BC,$0130       ; Counter = 304         
        LDIR

        LD      SP,$6FFC       ; Move Stack Pointer to Expanded Memory
        
        JP      L6000

;   Program Name
PRGNAM: DB     "CART TESTER"
LSTBYT: DB     $00             ; End


        ORG     $6000

L6000:  SYSTEM INTPC

        DO     SETOUT
        DB     176             ; Vertical Blanking Line
        DB     44              ; Left/Right Color Boundary
        DB     $08             ; Interrupt Mode
 
;  Setup Color Registers
        DO     COLSET
        DW     COLTAB          ; Color Table

;  Clear Screen
        DO     FILL
        DW     NORMEM          ; Destination
        DW     4000            ; Bytes to move
        DB     $00             ; Background color

;  Display "CART TEST PGM   SWAP CART NOW   PRESS = TO TEST"
        DO     STRDIS
        DB     $10             ; X coordinate
        DB     0               ; Y coordinate
        DB     $0C             ; Options
        DW     TSTPGM          ; Address of string to display

        EXIT

;   Wait for user to swap cart.  When the the cart has been swapped, 
;   the user will press "=" (or any key in the right-most colum)
;   and then the program will continue.
        DI                     ; Disabled for Cart swaps

L6010:  IN      A,($14)        ; Port 14 (Right-most Keypad Column)        
        OR      A              ; Check if Port $14 is equal to zero
        JR      Z,L6010        ; If Zero, then "=" not pressed, so
;                                loop until "=" pressed

        CALL    L60E7          ; Display Redo
        LD      DE,$7FFF       ; Restart (Beginning of Loop)
L601B:  INC     DE             ; Start at $8000
        LD      A,(DE)         ; Loading from RAM
        CPI                    ; Compare and Increment     
        JR      NZ,L6031       ; Not =, then error
        LD      A,B            ; 
        OR      C
        JR      NZ,L601B       ; If not done, then loop

; No Errors, Display "Good!" on screen
        SYSSUK  STRDIS         ;  UPI STRing DISplay
        DB      $10            ;  ... X = 16
        DB      $10            ;  ... Y = 16
        DB      $08            ;  ... Options = 8
        DW      GOOD           ;  ... String Address = 24704

L602C:  CALL    L60F0
        JR      L6000          ; Jump to Start of Program in RAM
        
L6031:  DEC     HL             ; Start of Error
        LD      BC,BADBYT+17   ; Loading with address of last dash 
        LD      A,L            ; Has address that is bad
        CALL    L60F9          ;  
        LD      BC,ISDASH+5    ; points to dashes after is
        LD      A,(HL)
        CALL    L6060
        LD      BC,SBDASH+7
        LD      A,(DE)         ; DE points to $8000 area
        CALL    L6060

        SYSSUK  STRDIS         ;  UPI STRing DISplay
        DB      $08            ;  ... X = 8
        DB      $10            ;  ... Y = 16
        DB      $04            ;  ... Options = 4
        DW      BADBYT         ;  ... String Address = 24710

        JR      L602C

        NOP     
        NOP     
        NOP     
        NOP     
        NOP     

L6053:  AND     $0F           ; Strips off high nibble
        CP      $0A           ; 
        JR      C,L605B       ; Less than?
        ADD     A,$07         
L605B:  ADD     A,$30         ; Puts in place of last dash- can do this
;                               because it is not run from cart-- but RAM! 
        LD      (BC),A
        DEC     BC
        RET     

; writing address to screen
L6060:  PUSH    AF             ; Saving
        CALL    L6053
        POP     AF             ; Recal
        SRL     A              ; puts highest to lowest 
        SRL     A               
        SRL     A
        SRL     A
        JR      L6053

        NOP

L6070:  SYSTEM  INTPC          ;  UPI INTerPret with Context create
;  Covers one line of text (swap cart now)
        DO      RECTAN         ;  UPI paint a RECTANgle
        DB      $00            ;  ... X = 0
        DB      $40            ;  ... Y = 64
        DB      $A0            ;  ... Width = 160
        DB      $08            ;  ... Height = 8
        DB      $00            ;  ... Color = 0
;  Display   
        DO      STRDIS         ;  UPI STRing DISplay
        DB      $38            ;  ... X = 56
        DB      $48            ;  ... Y = 72
        DB      $04            ;  ... Options = 4
        DW      REDO           ;  ... String Address = 24800

        DONT    XINTC          ;  UPI eXit INTerpreter with Context

        RET     

; Text

GOOD:   DB     "G00D!"
        DB     $00             ; End of String Marker

BADBYT: DB     "BAD BYTE @: %(----) "
        DB     $6B             ; Next 3 bytes...
        DB     $10             ;    control characters?
        DB     $18

ISDASH  DB     "IS: -- "
        DB     $6B             ; Next 2 bytes...
        DB     $08             ;    control characters?

SBDASH  DB     " S/B: --"
        DB     $00             ; End of String Marker

TSTPGM: DB     "CART TEST PGM"
        DB     $6B             ; Next 3 bytes...
        DB     $10             ;    control characters?
        DB     $40

        DB     "SWAP CART N0W"
        DB     $6B             ; Next 3 bytes
        DB     $08             ;   control characters?
        DB     $48

        DB     "PRESS = T0 TEST"
        DB     $00             ; End of String Marker

        NOP

REDO:   DB     $30             ; Next 2 bytes...
        DB     $04             ;   control characters? [tab over 4 spaces]
;  Note anything under 31 goes over that spaces...
        DB     "RED0"
        DB     $00             ; End of String Marker

L60E7:  CALL    L6070          
        LD      BC,$2000       ; Count counter for 8K of cart space
        PUSH    BC             ; 
        POP     HL             ; Put $2000 in HL
        
        RET     

; Print Text of of "wait" message  
L60F0:  CALL    L6070          ; 
L60F3:  IN      A,($16)        ; Port $16, Was 0 Pressed
        OR      A
        JR      Z,L60F3
        RET     

L60F9:  CALL    L6060          ; Put stuff on screen
        LD      A,H
        JP      L6060

;  8 Bytes COLTBL - Color Table for Color Register
;  Left side of Horizontal Color Boundary  
COLTAB: DB      $00            ; COL3L (Black) 
        DB      $8B            ; COL2L (Yellow/Brown
        DB      $52            ; COL1L (Red)  
        DB      $07            ; COL0L (White)  
;  Right side of Horizontal Color Boundary
        DB      $00            ; COL3R (Black) 
        DB      $8B            ; COL2R (Yellow/Brown 
        DB      $52            ; COL1R (Red)
        DB      $07            ; COL0R (White)           